// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: google/protobuf/compiler/java/java_features.proto
// Protobuf C++ Version: 7.34.2

#ifndef google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto_2epb_2eh
#define google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7034002
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_bases.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
#include "google/protobuf/descriptor.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto PROTOC_EXPORT

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct PROTOC_EXPORT TableStruct_google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
PROTOC_EXPORT extern const ::google::protobuf::internal::DescriptorTable descriptor_table_google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto;
}  // extern "C"
namespace pb {
enum JavaFeatures_NestInFileClassFeature_NestInFileClass : int;
PROTOC_EXPORT extern const uint32_t JavaFeatures_NestInFileClassFeature_NestInFileClass_internal_data_[];
enum JavaFeatures_Utf8Validation : int;
PROTOC_EXPORT extern const uint32_t JavaFeatures_Utf8Validation_internal_data_[];
class JavaFeatures;
struct JavaFeaturesDefaultTypeInternal;
PROTOC_EXPORT extern JavaFeaturesDefaultTypeInternal _JavaFeatures_default_instance_;
PROTOC_EXPORT extern const ::google::protobuf::internal::ClassDataFull JavaFeatures_class_data_;
class JavaFeatures_NestInFileClassFeature;
struct JavaFeatures_NestInFileClassFeatureDefaultTypeInternal;
PROTOC_EXPORT extern JavaFeatures_NestInFileClassFeatureDefaultTypeInternal _JavaFeatures_NestInFileClassFeature_default_instance_;
PROTOC_EXPORT extern const ::google::protobuf::internal::ClassDataFull JavaFeatures_NestInFileClassFeature_class_data_;
}  // namespace pb
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass_internal_data_>
    internal::EnumTraitsImpl::value<::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass>;
template <>
internal::EnumTraitsT<::pb::JavaFeatures_Utf8Validation_internal_data_>
    internal::EnumTraitsImpl::value<::pb::JavaFeatures_Utf8Validation>;
namespace internal {
template <>
inline ::absl::string_view GetFeatureSetDefaultsData<::pb::JavaFeatures>() {
  static constexpr char kDefaults[] = "CicYhAciA8o+ACodCAEQAhgCIAMoATACOAJAAco+CggBEAEYACABKAMKJxjnByIDyj4AKh0IAhABGAEgAigBMAE4AkAByj4KCAAQARgAIAEoAwonGOgHIhMIARABGAEgAigBMAHKPgQIABABKg04AkAByj4GGAAgASgDCicY6QciGwgBEAEYASACKAEwATgBQALKPggIABABGAAoASoFyj4CIAAg5gco6Qc=";
  return kDefaults;
}
}  // namespace internal
}  // namespace protobuf
}  // namespace google

namespace pb {
enum JavaFeatures_NestInFileClassFeature_NestInFileClass : int {
  JavaFeatures_NestInFileClassFeature_NestInFileClass_NEST_IN_FILE_CLASS_UNKNOWN = 0,
  JavaFeatures_NestInFileClassFeature_NestInFileClass_NO = 1,
  JavaFeatures_NestInFileClassFeature_NestInFileClass_YES = 2,
  JavaFeatures_NestInFileClassFeature_NestInFileClass_LEGACY = 3,
};

PROTOC_EXPORT extern const uint32_t JavaFeatures_NestInFileClassFeature_NestInFileClass_internal_data_[];
inline constexpr JavaFeatures_NestInFileClassFeature_NestInFileClass JavaFeatures_NestInFileClassFeature_NestInFileClass_NestInFileClass_MIN =
    static_cast<JavaFeatures_NestInFileClassFeature_NestInFileClass>(0);
inline constexpr JavaFeatures_NestInFileClassFeature_NestInFileClass JavaFeatures_NestInFileClassFeature_NestInFileClass_NestInFileClass_MAX =
    static_cast<JavaFeatures_NestInFileClassFeature_NestInFileClass>(3);
[[nodiscard]] inline bool JavaFeatures_NestInFileClassFeature_NestInFileClass_IsValid(int value) {
  return 0 <= value && value <= 3;
}
inline constexpr int JavaFeatures_NestInFileClassFeature_NestInFileClass_NestInFileClass_ARRAYSIZE = 3 + 1;
[[nodiscard]] PROTOC_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
JavaFeatures_NestInFileClassFeature_NestInFileClass_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(JavaFeatures_NestInFileClassFeature_NestInFileClass) {
  return JavaFeatures_NestInFileClassFeature_NestInFileClass_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& JavaFeatures_NestInFileClassFeature_NestInFileClass_Name(T value) {
  static_assert(::std::is_same<T, JavaFeatures_NestInFileClassFeature_NestInFileClass>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to NestInFileClass_Name().");
  return JavaFeatures_NestInFileClassFeature_NestInFileClass_Name(static_cast<JavaFeatures_NestInFileClassFeature_NestInFileClass>(value));
}
template <>
[[nodiscard]] inline const ::std::string& JavaFeatures_NestInFileClassFeature_NestInFileClass_Name(JavaFeatures_NestInFileClassFeature_NestInFileClass value) {
  return ::google::protobuf::internal::NameOfDenseEnum<JavaFeatures_NestInFileClassFeature_NestInFileClass_descriptor, 0, 3>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool JavaFeatures_NestInFileClassFeature_NestInFileClass_Parse(
    ::absl::string_view name, JavaFeatures_NestInFileClassFeature_NestInFileClass* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<JavaFeatures_NestInFileClassFeature_NestInFileClass>(JavaFeatures_NestInFileClassFeature_NestInFileClass_descriptor(), name,
                                           value);
}
enum JavaFeatures_Utf8Validation : int {
  JavaFeatures_Utf8Validation_UTF8_VALIDATION_UNKNOWN = 0,
  JavaFeatures_Utf8Validation_DEFAULT = 1,
  JavaFeatures_Utf8Validation_VERIFY = 2,
};

PROTOC_EXPORT extern const uint32_t JavaFeatures_Utf8Validation_internal_data_[];
inline constexpr JavaFeatures_Utf8Validation JavaFeatures_Utf8Validation_Utf8Validation_MIN =
    static_cast<JavaFeatures_Utf8Validation>(0);
inline constexpr JavaFeatures_Utf8Validation JavaFeatures_Utf8Validation_Utf8Validation_MAX =
    static_cast<JavaFeatures_Utf8Validation>(2);
[[nodiscard]] inline bool JavaFeatures_Utf8Validation_IsValid(int value) {
  return 0 <= value && value <= 2;
}
inline constexpr int JavaFeatures_Utf8Validation_Utf8Validation_ARRAYSIZE = 2 + 1;
[[nodiscard]] PROTOC_EXPORT const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
JavaFeatures_Utf8Validation_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(JavaFeatures_Utf8Validation) {
  return JavaFeatures_Utf8Validation_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& JavaFeatures_Utf8Validation_Name(T value) {
  static_assert(::std::is_same<T, JavaFeatures_Utf8Validation>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to Utf8Validation_Name().");
  return JavaFeatures_Utf8Validation_Name(static_cast<JavaFeatures_Utf8Validation>(value));
}
template <>
[[nodiscard]] inline const ::std::string& JavaFeatures_Utf8Validation_Name(JavaFeatures_Utf8Validation value) {
  return ::google::protobuf::internal::NameOfDenseEnum<JavaFeatures_Utf8Validation_descriptor, 0, 2>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool JavaFeatures_Utf8Validation_Parse(
    ::absl::string_view name, JavaFeatures_Utf8Validation* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<JavaFeatures_Utf8Validation>(JavaFeatures_Utf8Validation_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class PROTOC_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED JavaFeatures_NestInFileClassFeature final : public ::google::protobuf::internal::ZeroFieldsBase
/* @@protoc_insertion_point(class_definition:pb.JavaFeatures.NestInFileClassFeature) */ {
 public:
  inline JavaFeatures_NestInFileClassFeature() : JavaFeatures_NestInFileClassFeature(nullptr) {}

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(JavaFeatures_NestInFileClassFeature* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(JavaFeatures_NestInFileClassFeature));
  }
#endif

  template <typename = void>
  explicit constexpr JavaFeatures_NestInFileClassFeature(::google::protobuf::internal::ConstantInitialized);

  inline JavaFeatures_NestInFileClassFeature(const JavaFeatures_NestInFileClassFeature& from) : JavaFeatures_NestInFileClassFeature(nullptr, from) {}
  inline JavaFeatures_NestInFileClassFeature(JavaFeatures_NestInFileClassFeature&& from) noexcept
      : JavaFeatures_NestInFileClassFeature(nullptr, ::std::move(from)) {}
  inline JavaFeatures_NestInFileClassFeature& operator=(const JavaFeatures_NestInFileClassFeature& from) {
    CopyFrom(from);
    return *this;
  }
  inline JavaFeatures_NestInFileClassFeature& operator=(JavaFeatures_NestInFileClassFeature&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const JavaFeatures_NestInFileClassFeature& default_instance() {
    return *reinterpret_cast<const JavaFeatures_NestInFileClassFeature*>(
        &_JavaFeatures_NestInFileClassFeature_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(JavaFeatures_NestInFileClassFeature& a, JavaFeatures_NestInFileClassFeature& b) { a.Swap(&b); }
  inline void Swap(JavaFeatures_NestInFileClassFeature* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(JavaFeatures_NestInFileClassFeature* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] JavaFeatures_NestInFileClassFeature* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::internal::ZeroFieldsBase::DefaultConstruct<JavaFeatures_NestInFileClassFeature>(arena);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::CopyFrom;
  inline void CopyFrom(const JavaFeatures_NestInFileClassFeature& from) {
    ::google::protobuf::internal::ZeroFieldsBase::CopyImpl(*this, from);
  }
  using ::google::protobuf::internal::ZeroFieldsBase::MergeFrom;
  void MergeFrom(const JavaFeatures_NestInFileClassFeature& from) {
    ::google::protobuf::internal::ZeroFieldsBase::MergeImpl(*this, from);
  }

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "pb.JavaFeatures.NestInFileClassFeature"; }

  explicit JavaFeatures_NestInFileClassFeature(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  JavaFeatures_NestInFileClassFeature(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const JavaFeatures_NestInFileClassFeature& from);
  JavaFeatures_NestInFileClassFeature(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, JavaFeatures_NestInFileClassFeature&& from) noexcept
      : JavaFeatures_NestInFileClassFeature(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using NestInFileClass = JavaFeatures_NestInFileClassFeature_NestInFileClass;
  static constexpr NestInFileClass NEST_IN_FILE_CLASS_UNKNOWN = JavaFeatures_NestInFileClassFeature_NestInFileClass_NEST_IN_FILE_CLASS_UNKNOWN;
  static constexpr NestInFileClass NO = JavaFeatures_NestInFileClassFeature_NestInFileClass_NO;
  static constexpr NestInFileClass YES = JavaFeatures_NestInFileClassFeature_NestInFileClass_YES;
  static constexpr NestInFileClass LEGACY = JavaFeatures_NestInFileClassFeature_NestInFileClass_LEGACY;
  [[nodiscard]] static inline bool NestInFileClass_IsValid(int value) {
    return JavaFeatures_NestInFileClassFeature_NestInFileClass_IsValid(value);
  }
  static constexpr NestInFileClass NestInFileClass_MIN = JavaFeatures_NestInFileClassFeature_NestInFileClass_NestInFileClass_MIN;
  static constexpr NestInFileClass NestInFileClass_MAX = JavaFeatures_NestInFileClassFeature_NestInFileClass_NestInFileClass_MAX;
  static constexpr int NestInFileClass_ARRAYSIZE = JavaFeatures_NestInFileClassFeature_NestInFileClass_NestInFileClass_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL NestInFileClass_descriptor() {
    return JavaFeatures_NestInFileClassFeature_NestInFileClass_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& NestInFileClass_Name(T value) {
    return JavaFeatures_NestInFileClassFeature_NestInFileClass_Name(value);
  }
  [[nodiscard]] static inline bool NestInFileClass_Parse(
      ::absl::string_view name, NestInFileClass* PROTOBUF_NONNULL value) {
    return JavaFeatures_NestInFileClassFeature_NestInFileClass_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  // @@protoc_insertion_point(class_scope:pb.JavaFeatures.NestInFileClassFeature)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 0,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  friend struct ::TableStruct_google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto;
};

PROTOC_EXPORT extern const ::google::protobuf::internal::ClassDataFull JavaFeatures_NestInFileClassFeature_class_data_;
// -------------------------------------------------------------------

class PROTOC_EXPORT  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED JavaFeatures final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:pb.JavaFeatures) */ {
 public:
  inline JavaFeatures() : JavaFeatures(nullptr) {}
  ~JavaFeatures() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(JavaFeatures* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(JavaFeatures));
  }
#endif

  template <typename = void>
  explicit constexpr JavaFeatures(::google::protobuf::internal::ConstantInitialized);

  inline JavaFeatures(const JavaFeatures& from) : JavaFeatures(nullptr, from) {}
  inline JavaFeatures(JavaFeatures&& from) noexcept
      : JavaFeatures(nullptr, ::std::move(from)) {}
  inline JavaFeatures& operator=(const JavaFeatures& from) {
    CopyFrom(from);
    return *this;
  }
  inline JavaFeatures& operator=(JavaFeatures&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const JavaFeatures& default_instance() {
    return *reinterpret_cast<const JavaFeatures*>(
        &_JavaFeatures_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(JavaFeatures& a, JavaFeatures& b) { a.Swap(&b); }
  inline void Swap(JavaFeatures* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(JavaFeatures* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] JavaFeatures* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<JavaFeatures>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const JavaFeatures& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const JavaFeatures& from) { JavaFeatures::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(JavaFeatures* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "pb.JavaFeatures"; }

  explicit JavaFeatures(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  JavaFeatures(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const JavaFeatures& from);
  JavaFeatures(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, JavaFeatures&& from) noexcept
      : JavaFeatures(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using NestInFileClassFeature = JavaFeatures_NestInFileClassFeature;
  using Utf8Validation = JavaFeatures_Utf8Validation;
  static constexpr Utf8Validation UTF8_VALIDATION_UNKNOWN = JavaFeatures_Utf8Validation_UTF8_VALIDATION_UNKNOWN;
  static constexpr Utf8Validation DEFAULT = JavaFeatures_Utf8Validation_DEFAULT;
  static constexpr Utf8Validation VERIFY = JavaFeatures_Utf8Validation_VERIFY;
  [[nodiscard]] static inline bool Utf8Validation_IsValid(int value) {
    return JavaFeatures_Utf8Validation_IsValid(value);
  }
  static constexpr Utf8Validation Utf8Validation_MIN = JavaFeatures_Utf8Validation_Utf8Validation_MIN;
  static constexpr Utf8Validation Utf8Validation_MAX = JavaFeatures_Utf8Validation_Utf8Validation_MAX;
  static constexpr int Utf8Validation_ARRAYSIZE = JavaFeatures_Utf8Validation_Utf8Validation_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL Utf8Validation_descriptor() {
    return JavaFeatures_Utf8Validation_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& Utf8Validation_Name(T value) {
    return JavaFeatures_Utf8Validation_Name(value);
  }
  [[nodiscard]] static inline bool Utf8Validation_Parse(
      ::absl::string_view name, Utf8Validation* PROTOBUF_NONNULL value) {
    return JavaFeatures_Utf8Validation_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kUtf8ValidationFieldNumber = 2,
    kLegacyClosedEnumFieldNumber = 1,
    kLargeEnumFieldNumber = 3,
    kUseOldOuterClassnameDefaultFieldNumber = 4,
    kNestInFileClassFieldNumber = 5,
  };
  // optional .pb.JavaFeatures.Utf8Validation utf8_validation = 2 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_FIELD, targets = TARGET_TYPE_FILE, edition_defaults = {
  [[nodiscard]] bool has_utf8_validation()
      const;
  void clear_utf8_validation() ;
  [[nodiscard]] ::pb::JavaFeatures_Utf8Validation utf8_validation() const;
  void set_utf8_validation(::pb::JavaFeatures_Utf8Validation value);

  private:
  ::pb::JavaFeatures_Utf8Validation _internal_utf8_validation() const;
  void _internal_set_utf8_validation(::pb::JavaFeatures_Utf8Validation value);

  public:
  // optional bool legacy_closed_enum = 1 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_FIELD, targets = TARGET_TYPE_FILE, edition_defaults = {
  [[nodiscard]] bool has_legacy_closed_enum()
      const;
  void clear_legacy_closed_enum() ;
  [[nodiscard]] bool legacy_closed_enum() const;
  void set_legacy_closed_enum(bool value);

  private:
  bool _internal_legacy_closed_enum() const;
  void _internal_set_legacy_closed_enum(bool value);

  public:
  // optional bool large_enum = 3 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_ENUM, targets = TARGET_TYPE_FILE, edition_defaults = {
  [[nodiscard]] bool has_large_enum()
      const;
  void clear_large_enum() ;
  [[nodiscard]] bool large_enum() const;
  void set_large_enum(bool value);

  private:
  bool _internal_large_enum() const;
  void _internal_set_large_enum(bool value);

  public:
  // optional bool use_old_outer_classname_default = 4 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_FILE, edition_defaults = {
  [[nodiscard]] bool has_use_old_outer_classname_default()
      const;
  void clear_use_old_outer_classname_default() ;
  [[nodiscard]] bool use_old_outer_classname_default() const;
  void set_use_old_outer_classname_default(bool value);

  private:
  bool _internal_use_old_outer_classname_default() const;
  void _internal_set_use_old_outer_classname_default(bool value);

  public:
  // optional .pb.JavaFeatures.NestInFileClassFeature.NestInFileClass nest_in_file_class = 5 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_MESSAGE, targets = TARGET_TYPE_ENUM, targets = TARGET_TYPE_SERVICE, edition_defaults = {
  [[nodiscard]] bool has_nest_in_file_class()
      const;
  void clear_nest_in_file_class() ;
  [[nodiscard]] ::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass nest_in_file_class() const;
  void set_nest_in_file_class(::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass value);

  private:
  ::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass _internal_nest_in_file_class() const;
  void _internal_set_nest_in_file_class(::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass value);

  public:
  // @@protoc_insertion_point(class_scope:pb.JavaFeatures)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 5,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const JavaFeatures& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    int utf8_validation_;
    bool legacy_closed_enum_;
    bool large_enum_;
    bool use_old_outer_classname_default_;
    int nest_in_file_class_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto;
};

PROTOC_EXPORT extern const ::google::protobuf::internal::ClassDataFull JavaFeatures_class_data_;

// ===================================================================



inline constexpr int kJavaFieldNumber = 1001;
PROTOC_EXPORT extern ::google::protobuf::internal::ExtensionIdentifier<
    ::google::protobuf::FeatureSet, ::google::protobuf::internal::MessageTypeTraits< ::pb::JavaFeatures >, 11, false>
    java;

// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// JavaFeatures_NestInFileClassFeature

// -------------------------------------------------------------------

// JavaFeatures

// optional bool legacy_closed_enum = 1 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_FIELD, targets = TARGET_TYPE_FILE, edition_defaults = {
inline bool JavaFeatures::has_legacy_closed_enum() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void JavaFeatures::clear_legacy_closed_enum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.legacy_closed_enum_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline bool JavaFeatures::legacy_closed_enum() const {
  // @@protoc_insertion_point(field_get:pb.JavaFeatures.legacy_closed_enum)
  return _internal_legacy_closed_enum();
}
inline void JavaFeatures::set_legacy_closed_enum(bool value) {
  _internal_set_legacy_closed_enum(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:pb.JavaFeatures.legacy_closed_enum)
}
inline bool JavaFeatures::_internal_legacy_closed_enum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.legacy_closed_enum_;
}
inline void JavaFeatures::_internal_set_legacy_closed_enum(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.legacy_closed_enum_ = value;
}

// optional .pb.JavaFeatures.Utf8Validation utf8_validation = 2 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_FIELD, targets = TARGET_TYPE_FILE, edition_defaults = {
inline bool JavaFeatures::has_utf8_validation() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void JavaFeatures::clear_utf8_validation() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.utf8_validation_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::pb::JavaFeatures_Utf8Validation JavaFeatures::utf8_validation() const {
  // @@protoc_insertion_point(field_get:pb.JavaFeatures.utf8_validation)
  return _internal_utf8_validation();
}
inline void JavaFeatures::set_utf8_validation(::pb::JavaFeatures_Utf8Validation value) {
  _internal_set_utf8_validation(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:pb.JavaFeatures.utf8_validation)
}
inline ::pb::JavaFeatures_Utf8Validation JavaFeatures::_internal_utf8_validation() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::pb::JavaFeatures_Utf8Validation>(_impl_.utf8_validation_);
}
inline void JavaFeatures::_internal_set_utf8_validation(::pb::JavaFeatures_Utf8Validation value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::pb::JavaFeatures_Utf8Validation_internal_data_));
                                          _impl_.utf8_validation_ = value;
}

// optional bool large_enum = 3 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_ENUM, targets = TARGET_TYPE_FILE, edition_defaults = {
inline bool JavaFeatures::has_large_enum() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void JavaFeatures::clear_large_enum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.large_enum_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline bool JavaFeatures::large_enum() const {
  // @@protoc_insertion_point(field_get:pb.JavaFeatures.large_enum)
  return _internal_large_enum();
}
inline void JavaFeatures::set_large_enum(bool value) {
  _internal_set_large_enum(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:pb.JavaFeatures.large_enum)
}
inline bool JavaFeatures::_internal_large_enum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.large_enum_;
}
inline void JavaFeatures::_internal_set_large_enum(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.large_enum_ = value;
}

// optional bool use_old_outer_classname_default = 4 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_FILE, edition_defaults = {
inline bool JavaFeatures::has_use_old_outer_classname_default() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void JavaFeatures::clear_use_old_outer_classname_default() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.use_old_outer_classname_default_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline bool JavaFeatures::use_old_outer_classname_default() const {
  // @@protoc_insertion_point(field_get:pb.JavaFeatures.use_old_outer_classname_default)
  return _internal_use_old_outer_classname_default();
}
inline void JavaFeatures::set_use_old_outer_classname_default(bool value) {
  _internal_set_use_old_outer_classname_default(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:pb.JavaFeatures.use_old_outer_classname_default)
}
inline bool JavaFeatures::_internal_use_old_outer_classname_default() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.use_old_outer_classname_default_;
}
inline void JavaFeatures::_internal_set_use_old_outer_classname_default(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.use_old_outer_classname_default_ = value;
}

// optional .pb.JavaFeatures.NestInFileClassFeature.NestInFileClass nest_in_file_class = 5 [retention = RETENTION_RUNTIME, targets = TARGET_TYPE_MESSAGE, targets = TARGET_TYPE_ENUM, targets = TARGET_TYPE_SERVICE, edition_defaults = {
inline bool JavaFeatures::has_nest_in_file_class() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void JavaFeatures::clear_nest_in_file_class() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nest_in_file_class_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass JavaFeatures::nest_in_file_class() const {
  // @@protoc_insertion_point(field_get:pb.JavaFeatures.nest_in_file_class)
  return _internal_nest_in_file_class();
}
inline void JavaFeatures::set_nest_in_file_class(::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass value) {
  _internal_set_nest_in_file_class(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:pb.JavaFeatures.nest_in_file_class)
}
inline ::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass JavaFeatures::_internal_nest_in_file_class() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass>(_impl_.nest_in_file_class_);
}
inline void JavaFeatures::_internal_set_nest_in_file_class(::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass_internal_data_));
                                          _impl_.nest_in_file_class_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace pb


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass>() {
  return ::pb::JavaFeatures_NestInFileClassFeature_NestInFileClass_descriptor();
}
template <>
struct is_proto_enum<::pb::JavaFeatures_Utf8Validation> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::pb::JavaFeatures_Utf8Validation>() {
  return ::pb::JavaFeatures_Utf8Validation_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // google_2fprotobuf_2fcompiler_2fjava_2fjava_5ffeatures_2eproto_2epb_2eh
